@@ -1,10 +1,14 @@
 from __future__ import annotations
 
+from typing import TYPE_CHECKING
+
 import pytest
-from pytest_benchmark.fixture import BenchmarkFixture
 
 import ry
 
+if TYPE_CHECKING:
+    from pytest_benchmark.fixture import BenchmarkFixture
+
 
 @pytest.mark.benchmark(group="jiff-constructors")
 def test_fn_datetime(benchmark: BenchmarkFixture):
