@@ -1,5 +1,6 @@
+mod byte;
 mod pathlike;
 mod string_or_strings;
-
+pub use byte::Byte;
 pub use pathlike::PathLike;
 pub use string_or_strings::StringOrStrings;
