@@ -2,7 +2,9 @@ use pyo3::intern;
 use pyo3::types::PyTuple;
 use pyo3::{exceptions::PyDeprecationWarning, prelude::*};
 
+// TODO: remove this
 /// Raise Deprecation warning for `intz`
+#[allow(dead_code)]
 pub(crate) fn deprecation_warning_intz(py: Python) -> PyResult<()> {
     let warnings_mod = py.import(intern!(py, "warnings"))?;
     let warning = PyDeprecationWarning::new_err("`intz` deprecated use `in_tz` instead");
