@@ -17,5 +17,5 @@ mod zoned;
 pub use date::*;
 pub use signed_duration::*;
 pub use time::*;
-pub use timezone::timezone2pyobect;
-pub use zoned::zoned2pyobect;
+pub use timezone::timezone2pyobject;
+pub use zoned::zoned2pyobject;
