@@ -1,13 +1,15 @@
 use pyo3::prelude::*;
 
-#[pyclass(name = "Weekday", module = "ry.ryo3", frozen)]
+#[pyclass(name = "Weekday", frozen)]
+#[cfg_attr(feature = "ry", pyo3(module = "ry.ryo3"))]
 #[derive(Debug, Clone, Copy, PartialEq)]
 pub struct RyWeekday(pub(crate) jiff::civil::Weekday);
 
 #[pymethods]
 impl RyWeekday {
     #[expect(clippy::trivially_copy_pass_by_ref)]
-    fn string(&self) -> &'static str {
+    #[pyo3(name = "to_string")]
+    fn py_to_string(&self) -> &'static str {
         match self.0 {
             jiff::civil::Weekday::Sunday => "sunday",
             jiff::civil::Weekday::Monday => "monday",
