@@ -1,26 +1,27 @@
-use crate::deprecations::deprecation_warning_intz;
-use crate::errors::{map_py_overflow_err, map_py_value_err};
+use crate::errors::{map_py_overflow_err, map_py_runtime_err, map_py_value_err};
+use crate::isoformat::{ISOFORMAT_PRINTER, ISOFORMAT_PRINTER_NO_MICROS};
+use crate::round::RyZonedDateTimeRound;
 use crate::ry_datetime::RyDateTime;
 use crate::ry_iso_week_date::RyISOWeekDate;
-use crate::ry_offset::RyOffset;
+use crate::ry_offset::{RyOffset, print_isoformat_offset};
 use crate::ry_signed_duration::RySignedDuration;
 use crate::ry_span::RySpan;
 use crate::ry_time::RyTime;
 use crate::ry_timestamp::RyTimestamp;
 use crate::ry_timezone::RyTimeZone;
-use crate::ry_zoned_round::RyZonedDateTimeRound;
 use crate::spanish::Spanish;
 use crate::{
     JiffEra, JiffEraYear, JiffRoundMode, JiffTzDisambiguation, JiffTzOffsetConflict, JiffUnit,
     JiffWeekday, JiffZoned, RyDate,
 };
 use jiff::civil::{Date, Time, Weekday};
-use jiff::tz::TimeZone;
+use jiff::tz::{Offset, TimeZone};
 use jiff::{Zoned, ZonedDifference, ZonedRound};
 use pyo3::IntoPyObjectExt;
 use pyo3::prelude::*;
 use pyo3::pyclass::CompareOp;
-use pyo3::types::{PyTuple, PyType};
+use pyo3::types::{PyDict, PyTuple};
+use ryo3_macro_rules::{any_repr, py_type_err};
 use std::fmt::Display;
 use std::hash::{DefaultHasher, Hash, Hasher};
 use std::str::FromStr;
@@ -28,7 +29,8 @@ use std::str::FromStr;
 #[cfg_attr(feature = "serde", derive(serde::Serialize))]
 #[cfg_attr(feature = "serde", serde(transparent))]
 #[derive(Debug, Clone, PartialEq)]
-#[pyclass(name = "ZonedDateTime", module = "ry.ryo3", frozen)]
+#[pyclass(name = "ZonedDateTime", frozen)]
+#[cfg_attr(feature = "ry", pyo3(module = "ry.ryo3"))]
 pub struct RyZoned(pub(crate) Zoned);
 
 #[pymethods]
@@ -80,9 +82,9 @@ impl RyZoned {
         )
     }
 
-    #[classmethod]
+    #[staticmethod]
     #[pyo3(signature = (tz=None))]
-    fn now(_cls: &Bound<'_, PyType>, tz: Option<&str>) -> PyResult<Self> {
+    fn now(tz: Option<&str>) -> PyResult<Self> {
         if let Some(tz) = tz {
             Zoned::now()
                 .in_tz(tz)
@@ -93,67 +95,98 @@ impl RyZoned {
         }
     }
 
-    #[classmethod]
-    fn utcnow(_cls: &Bound<'_, PyType>) -> PyResult<Self> {
-        Zoned::now()
-            .in_tz("UTC")
-            .map(Self::from)
-            .map_err(map_py_value_err)
+    #[staticmethod]
+    pub(crate) fn utcnow() -> Self {
+        Self::from(Zoned::now().with_time_zone(TimeZone::UTC))
     }
-
-    #[classmethod]
+    #[staticmethod]
     #[pyo3(signature = (timestamp, time_zone))]
-    fn from_parts(
-        _cls: &Bound<'_, PyType>,
-        timestamp: &RyTimestamp,
-        time_zone: &RyTimeZone,
-    ) -> Self {
+    fn from_parts(timestamp: &RyTimestamp, time_zone: &RyTimeZone) -> Self {
         let ts = timestamp.0;
         Self::from(Zoned::new(ts, time_zone.into()))
     }
 
-    #[classmethod]
-    fn parse(_cls: &Bound<'_, PyType>, s: &str) -> PyResult<Self> {
+    #[staticmethod]
+    fn from_str(s: &str) -> PyResult<Self> {
         Zoned::from_str(s).map(Self::from).map_err(map_py_value_err)
     }
 
-    #[classmethod]
-    fn strptime(_cls: &Bound<'_, PyType>, format: &str, input: &str) -> PyResult<Self> {
-        Zoned::strptime(format, input)
+    #[staticmethod]
+    fn parse(input: &str) -> PyResult<Self> {
+        Self::from_str(input)
+    }
+
+    // ========================================================================
+    // STRPTIME/STRFTIME
+    // ========================================================================
+    // Orig version but idk if it is needed....
+    // fn __format__(&self, fmt: &str) -> String {
+    //     self.0.strftime(fmt).to_string()
+    // }
+
+    // fn strftime(&self, fmt: &str) -> String {
+    //     self.0.strftime(fmt).to_string()
+    // }
+
+    // NOT SURE THIS IS NEEDED AS AFAICT jiff::Zoned doesn't fail?
+    fn __format__(&self, fmt: &str) -> PyResult<String> {
+        self.strftime(fmt)
+    }
+
+    #[pyo3(signature = (fmt))]
+    fn strftime(&self, fmt: &str) -> PyResult<String> {
+        let bdt: jiff::fmt::strtime::BrokenDownTime = (&self.0).into();
+        bdt.to_string(fmt).map_err(map_py_value_err)
+    }
+
+    #[staticmethod]
+    #[pyo3(signature = (s, /, fmt))]
+    fn strptime(s: &str, fmt: &str) -> PyResult<Self> {
+        Zoned::strptime(fmt, s)
             .map(Self::from)
             .map_err(map_py_value_err)
     }
 
-    #[classmethod]
-    fn parse_rfc2822(_cls: &Bound<'_, PyType>, input: &str) -> PyResult<Self> {
+    #[staticmethod]
+    fn parse_rfc2822(input: &str) -> PyResult<Self> {
         ::jiff::fmt::rfc2822::parse(input)
             .map(Self::from)
-            .map_err(|e| PyErr::new::<pyo3::exceptions::PyValueError, _>(format!("{e}")))
+            .map_err(map_py_value_err)
     }
 
     #[staticmethod]
     fn from_rfc2822(s: &str) -> PyResult<Self> {
         jiff::fmt::rfc2822::parse(s)
             .map(Self::from)
-            .map_err(|e| PyErr::new::<pyo3::exceptions::PyValueError, _>(format!("{e}")))
-    }
-
-    fn strftime(&self, format: &str) -> String {
-        self.0.strftime(format).to_string()
+            .map_err(map_py_value_err)
     }
 
     fn format_rfc2822(&self) -> PyResult<String> {
-        jiff::fmt::rfc2822::to_string(&self.0)
-            .map_err(|e| PyErr::new::<pyo3::exceptions::PyValueError, _>(format!("{e}")))
+        jiff::fmt::rfc2822::to_string(&self.0).map_err(map_py_value_err)
     }
 
     fn to_rfc2822(&self) -> PyResult<String> {
-        jiff::fmt::rfc2822::to_string(&self.0)
-            .map_err(|e| PyErr::new::<pyo3::exceptions::PyValueError, _>(format!("{e}")))
+        jiff::fmt::rfc2822::to_string(&self.0).map_err(map_py_value_err)
     }
-
-    fn isoformat(&self) -> String {
-        self.0.datetime().to_string()
+    // ISO format mismatch:
+    // input datetime: 7639-01-01 00:00:00.395000+00:00 (repr: datetime.datetime(7639, 1, 1, 0, 0, 0, 395000, tzinfo=zoneinfo.ZoneInfo(key='UTC')))
+    // py: 7639-01-01T00:00:00.395000+00:00
+    // ry: 7639-01-01T00:00:00+00
+    // is_eq: False
+    // ry_prefix_ok: False
+    fn isoformat(&self) -> PyResult<String> {
+        let offset: Offset = self.0.offset();
+        // let ts = self.0.timestamp();
+        let dattie = self.0.datetime();
+        let mut s = String::with_capacity(32);
+        if self.0.datetime().microsecond() == 0 && self.0.subsec_nanosecond() == 0 {
+            ISOFORMAT_PRINTER_NO_MICROS.print_datetime(&dattie, &mut s)
+        } else {
+            ISOFORMAT_PRINTER.print_datetime(&dattie, &mut s)
+        }
+        .map_err(map_py_runtime_err)?;
+        print_isoformat_offset(&offset, &mut s).map_err(map_py_runtime_err)?;
+        Ok(s)
     }
 
     fn __richcmp__(&self, other: &Self, op: CompareOp) -> bool {
@@ -167,8 +200,9 @@ impl RyZoned {
         }
     }
 
-    fn string(&self) -> String {
-        self.0.to_string()
+    #[pyo3(name = "to_string")]
+    fn py_to_string(&self) -> String {
+        self.__str__()
     }
 
     fn __str__(&self) -> String {
@@ -176,26 +210,29 @@ impl RyZoned {
     }
 
     fn __repr__(&self) -> String {
-        // representable format
-        format!("ZonedDateTime.parse(\"{}\")", self.0)
+        format!("{self}")
     }
 
     fn __hash__(&self) -> u64 {
         let mut hasher = DefaultHasher::new();
-        self.0.datetime().hash(&mut hasher);
+        self.0.hash(&mut hasher);
+        self.0
+            .time_zone()
+            .to_offset_info(self.0.timestamp())
+            .hash(&mut hasher);
         self.0.time_zone().iana_name().hash(&mut hasher);
         hasher.finish()
     }
 
-    fn timestamp(&self) -> RyTimestamp {
+    pub(crate) fn timestamp(&self) -> RyTimestamp {
         RyTimestamp::from(self.0.timestamp())
     }
 
-    fn date(&self) -> RyDate {
+    pub(crate) fn date(&self) -> RyDate {
         RyDate::from(self.0.date())
     }
 
-    fn time(&self) -> RyTime {
+    pub(crate) fn time(&self) -> RyTime {
         RyTime::from(self.0.time())
     }
 
@@ -203,6 +240,23 @@ impl RyZoned {
         RyDateTime::from(self.0.datetime())
     }
 
+    fn to_dict<'py>(&self, py: Python<'py>) -> PyResult<Bound<'py, PyDict>> {
+        use crate::interns;
+        let dict = PyDict::new(py);
+        dict.set_item(interns::year(py), self.0.year())?;
+        dict.set_item(interns::month(py), self.0.month())?;
+        dict.set_item(interns::day(py), self.0.day())?;
+        dict.set_item(interns::hour(py), self.0.hour())?;
+        dict.set_item(interns::minute(py), self.0.minute())?;
+        dict.set_item(interns::second(py), self.0.second())?;
+        dict.set_item(interns::nanosecond(py), self.0.subsec_nanosecond())?;
+        dict.set_item(
+            interns::tz(py),
+            self.0.time_zone().iana_name().unwrap_or("unknown"),
+        )?;
+        Ok(dict)
+    }
+
     fn to_py(&self) -> &Zoned {
         self.to_pydatetime()
     }
@@ -223,20 +277,22 @@ impl RyZoned {
         self.0.time_zone()
     }
 
-    #[classmethod]
-    fn from_pydatetime(_cls: &Bound<'_, PyType>, d: JiffZoned) -> Self {
+    #[staticmethod]
+    fn from_pydatetime(d: JiffZoned) -> Self {
         Self::from(d.0)
     }
 
     fn in_tz(&self, tz: &str) -> PyResult<Self> {
-        self.0
-            .in_tz(tz)
-            .map(Self::from)
-            .map_err(|e| PyErr::new::<pyo3::exceptions::PyValueError, _>(format!("{e}")))
+        self.0.in_tz(tz).map(Self::from).map_err(map_py_value_err)
     }
 
-    fn intz(&self, py: Python, tz: &str) -> PyResult<Self> {
-        deprecation_warning_intz(py)?;
+    #[pyo3(
+        warn(
+            message = "`intz` is deprecated, use `in_tz` instead",
+            category = pyo3::exceptions::PyDeprecationWarning
+        )
+    )]
+    fn intz(&self, tz: &str) -> PyResult<Self> {
         self.in_tz(tz)
     }
 
@@ -244,11 +300,8 @@ impl RyZoned {
         self.in_tz(tz)
     }
 
-    fn inutc(&self) -> PyResult<Self> {
-        self.0
-            .in_tz("UTC")
-            .map(Self::from)
-            .map_err(|e| PyErr::new::<pyo3::exceptions::PyValueError, _>(format!("{e}")))
+    fn inutc(&self) -> Self {
+        Self::from(self.0.with_time_zone(TimeZone::UTC))
     }
 
     fn __sub__<'py>(
@@ -257,7 +310,7 @@ impl RyZoned {
         other: &Bound<'py, PyAny>,
     ) -> PyResult<Bound<'py, PyAny>> {
         #[expect(clippy::arithmetic_side_effects)]
-        if let Ok(zoned) = other.downcast::<Self>() {
+        if let Ok(zoned) = other.cast::<Self>() {
             // if other is a Zoned, return a Span
             let span = &self.0 - &zoned.get().0;
             let obj = RySpan::from(span).into_pyobject(py).map(Bound::into_any)?;
@@ -277,15 +330,11 @@ impl RyZoned {
             .map_err(map_py_overflow_err)
     }
 
-    fn checked_add<'py>(&self, other: &'py Bound<'py, PyAny>) -> PyResult<Self> {
+    fn add<'py>(&self, other: &'py Bound<'py, PyAny>) -> PyResult<Self> {
         self.__add__(other)
     }
 
-    fn checked_sub<'py>(
-        &self,
-        py: Python<'py>,
-        other: &Bound<'py, PyAny>,
-    ) -> PyResult<Bound<'py, PyAny>> {
+    fn sub<'py>(&self, py: Python<'py>, other: &Bound<'py, PyAny>) -> PyResult<Bound<'py, PyAny>> {
         self.__sub__(py, other)
     }
 
@@ -321,14 +370,11 @@ impl RyZoned {
         self.0
             .round(zdt_round)
             .map(Self::from)
-            .map_err(|e| PyErr::new::<pyo3::exceptions::PyValueError, _>(format!("{e}")))
+            .map_err(map_py_value_err)
     }
 
-    fn _round(&self, dt_round: &RyZonedDateTimeRound) -> PyResult<Self> {
-        self.0
-            .round(dt_round.round)
-            .map(Self::from)
-            .map_err(|e| PyErr::new::<pyo3::exceptions::PyValueError, _>(format!("{e}")))
+    fn _round(&self, zdt_round: &RyZonedDateTimeRound) -> PyResult<Self> {
+        zdt_round.round(self)
     }
 
     fn tomorrow(&self) -> PyResult<Self> {
@@ -553,21 +599,19 @@ impl RyZoned {
         let mut builder = self.0.with();
         if let Some(obj) = obj {
             // if obj is a Zoned, use it as the base
-            if let Ok(zoned) = obj.downcast::<RyDate>() {
+            if let Ok(zoned) = obj.cast::<RyDate>() {
                 // if obj is a Zoned, use it as the base
                 let date = zoned.extract::<RyDate>()?;
                 builder = builder.date(date.0);
-            } else if let Ok(time) = obj.downcast::<RyTime>() {
+            } else if let Ok(time) = obj.cast::<RyTime>() {
                 // if obj is a Time, use it as the base
                 let time = time.extract::<RyTime>()?;
                 builder = builder.time(time.0);
-            } else if let Ok(date) = obj.downcast::<RyOffset>() {
+            } else if let Ok(date) = obj.cast::<RyOffset>() {
                 let offset = date.extract::<RyOffset>()?;
                 builder = builder.offset(offset.0);
             } else {
-                return Err(PyErr::new::<pyo3::exceptions::PyTypeError, _>(format!(
-                    "obj must be a Date, Time or Offset; given: {obj}",
-                )));
+                return py_type_err!("obj must be a Date, Time or Offset; given: {obj}",);
             }
         }
 
@@ -705,13 +749,74 @@ impl RyZoned {
     fn subsec_nanosecond(&self) -> i32 {
         self.0.subsec_nanosecond()
     }
+
+    #[staticmethod]
+    fn from_any<'py>(value: &Bound<'py, PyAny>) -> PyResult<Bound<'py, PyAny>> {
+        let py = value.py();
+        if let Ok(pystr) = value.cast::<pyo3::types::PyString>() {
+            let s = pystr.extract::<&str>()?;
+            Self::from_str(s).map(|dt| dt.into_bound_py_any(py).map(Bound::into_any))?
+        } else if let Ok(pybytes) = value.cast::<pyo3::types::PyBytes>() {
+            let s = String::from_utf8_lossy(pybytes.as_bytes());
+            Self::from_str(&s).map(|dt| dt.into_bound_py_any(py).map(Bound::into_any))?
+        } else if value.is_exact_instance_of::<Self>() {
+            value.into_bound_py_any(py)
+        } else if let Ok(d) = value.cast_exact::<RyTimestamp>() {
+            let dt = d.get().0.to_zoned(TimeZone::UTC);
+            dt.into_bound_py_any(py)
+        } else if let Ok(d) = value.extract::<JiffZoned>() {
+            Self::from(d.0).into_bound_py_any(py)
+        } else {
+            let valtype = any_repr!(value);
+            py_type_err!("ZonedDateTime conversion error: {valtype}",)
+        }
+    }
+    // ========================================================================
+    // PYDANTIC
+    // ========================================================================
+
+    #[cfg(feature = "pydantic")]
+    #[staticmethod]
+    fn _pydantic_validate<'py>(
+        value: &Bound<'py, PyAny>,
+        _handler: &Bound<'py, PyAny>,
+    ) -> PyResult<Bound<'py, PyAny>> {
+        Self::from_any(value).map_err(map_py_value_err)
+    }
+
+    #[cfg(feature = "pydantic")]
+    #[classmethod]
+    fn __get_pydantic_core_schema__<'py>(
+        cls: &Bound<'py, ::pyo3::types::PyType>,
+        source: &Bound<'py, PyAny>,
+        handler: &Bound<'py, PyAny>,
+    ) -> PyResult<Bound<'py, PyAny>> {
+        use ryo3_pydantic::GetPydanticCoreSchemaCls;
+        Self::get_pydantic_core_schema(cls, source, handler)
+    }
 }
 
 impl Display for RyZoned {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        write!(f, "{}", self.0)
+        if let Some(tz_name) = self.0.time_zone().iana_name() {
+            write!(
+                f,
+                "ZonedDateTime(year={}, month={}, day={}, hour={}, minute={}, second={}, nanosecond={}, tz=\"{}\")",
+                self.0.year(),
+                self.0.month(),
+                self.0.day(),
+                self.0.hour(),
+                self.0.minute(),
+                self.0.second(),
+                self.0.subsec_nanosecond(),
+                tz_name
+            )
+        } else {
+            write!(f, "ZonedDateTime.parse(\"{}\")", self.0)
+        }
     }
 }
+
 impl From<Zoned> for RyZoned {
     fn from(value: Zoned) -> Self {
         Self(value)
