@@ -1,7 +1,7 @@
 use crate::RyResponse;
 use crate::errors::map_reqwest_err;
-use crate::query_like::QueryLike;
 use crate::user_agent::parse_user_agent;
+use bytes::Bytes;
 use pyo3::exceptions::PyValueError;
 use pyo3::prelude::*;
 use pyo3::types::{PyDict, PyTuple};
@@ -9,12 +9,13 @@ use pyo3::{IntoPyObjectExt, intern};
 use reqwest::header::HeaderMap;
 use reqwest::{Method, RequestBuilder};
 use ryo3_http::{HttpVersion, PyHeaders, PyHeadersLike};
-use ryo3_macro_rules::err_py_not_impl;
+use ryo3_macro_rules::{py_type_err, py_value_err, pytodo};
+use ryo3_std::time::PyDuration;
 use ryo3_url::extract_url;
-use tracing::debug;
 
 #[derive(Debug, Clone)]
-#[pyclass(name = "HttpClient", module = "ry.ryo3.reqwest", frozen)]
+#[pyclass(name = "HttpClient", frozen)]
+#[cfg_attr(feature = "ry", pyo3(module = "ry.ryo3"))]
 pub struct RyHttpClient {
     client: reqwest::Client,
     cfg: ClientConfig,
@@ -26,26 +27,27 @@ pub struct ClientConfig {
     headers: Option<PyHeaders>,
     cookies: bool,
     user_agent: Option<ryo3_http::HttpHeaderValue>,
-    timeout: Option<ryo3_std::PyDuration>,
-    read_timeout: Option<ryo3_std::PyDuration>,
-    connect_timeout: Option<ryo3_std::PyDuration>,
+    timeout: Option<PyDuration>,
+    read_timeout: Option<PyDuration>,
+    connect_timeout: Option<PyDuration>,
     gzip: bool,
     brotli: bool,
     deflate: bool,
     zstd: bool,
+    hickory_dns: bool,
     http1_only: bool,
 }
 
 struct RequestKwargs<'py> {
     url: &'py Bound<'py, PyAny>,
     method: Method,
-    body: Option<ryo3_bytes::PyBytes>,
+    body: Option<&'py Bound<'py, PyAny>>,
     headers: Option<PyHeadersLike>,
     query: Option<&'py Bound<'py, PyAny>>,
     json: Option<&'py Bound<'py, PyAny>>,
     multipart: Option<&'py Bound<'py, PyAny>>,
     form: Option<&'py Bound<'py, PyAny>>,
-    timeout: Option<&'py ryo3_std::PyDuration>,
+    timeout: Option<&'py PyDuration>,
     version: Option<HttpVersion>,
 }
 
@@ -73,24 +75,43 @@ impl RyHttpClient {
             req = req.timeout(timeout.0);
         }
         if let Some(query) = options.query {
-            let q = QueryLike::extract_bound(query)?;
-            req = req.query(&q);
+            let pyser = ryo3_serde::SerializePyAny::new(query, None);
+            req = req.query(&pyser);
+        }
+
+        // make sure only one of body, json, form, multipart is set
+        if u8::from(options.body.is_some())
+            + u8::from(options.json.is_some())
+            + u8::from(options.form.is_some())
+            + u8::from(options.multipart.is_some())
+            > 1
+        {
+            return py_value_err!("body, json, form, multipart are mutually exclusive");
+        }
+
+        if let Some(_multipart) = options.multipart {
+            pytodo!("multipart not implemented (yet)");
         }
         if let Some(json) = options.json {
             let wrapped = ryo3_serde::SerializePyAny::new(json, None);
             req = req.json(&wrapped);
         }
-        if let Some(_multipart) = options.multipart {
-            return err_py_not_impl!("multipart not implemented (yet)");
-        }
-        if let Some(_form) = options.form {
-            return err_py_not_impl!("form not implemented (yet)");
+        if let Some(form) = options.form {
+            let pyser = ryo3_serde::SerializePyAny::new(form, None);
+            req = req.form(&pyser);
         }
         if let Some(body) = options.body {
-            let body_bytes = body.into_inner();
-            req = req.body(body_bytes);
+            if let Ok(rsbytes) = body.cast_exact::<ryo3_bytes::PyBytes>() {
+                // short circuit for rs-py-bytes
+                let rsbytes: &Bytes = rsbytes.get().as_ref();
+                req = req.body(rsbytes.to_owned());
+            } else if let Ok(bytes) = body.extract::<ryo3_bytes::PyBytes>() {
+                // buffer protocol
+                req = req.body(bytes.into_inner());
+            } else {
+                return py_type_err!("body must be bytes-like or string");
+            }
         }
-        debug!("reqwest-client-fetch: {:#?}", req);
         Ok(req)
     }
 }
@@ -112,6 +133,7 @@ impl RyHttpClient {
             brotli = true,
             deflate = true,
             zstd = true,
+            hickory_dns = true,
             http1_only = false,
         )
     )]
@@ -119,13 +141,14 @@ impl RyHttpClient {
         headers: Option<PyHeadersLike>,
         cookies: bool,
         user_agent: Option<String>,
-        timeout: Option<ryo3_std::PyDuration>,
-        read_timeout: Option<ryo3_std::PyDuration>,
-        connect_timeout: Option<ryo3_std::PyDuration>,
+        timeout: Option<PyDuration>,
+        read_timeout: Option<PyDuration>,
+        connect_timeout: Option<PyDuration>,
         gzip: Option<bool>,
         brotli: Option<bool>,
         deflate: Option<bool>,
         zstd: Option<bool>,
+        hickory_dns: Option<bool>,
         http1_only: Option<bool>,
     ) -> PyResult<Self> {
         let user_agent = parse_user_agent(user_agent)?;
@@ -141,9 +164,9 @@ impl RyHttpClient {
             brotli: brotli.unwrap_or(true),
             deflate: deflate.unwrap_or(true),
             zstd: zstd.unwrap_or(true),
+            hickory_dns: hickory_dns.unwrap_or(true),
             http1_only: http1_only.unwrap_or(false),
         };
-        debug!("reqwest-client-config: {:#?}", client_cfg);
         let client_builder = client_cfg.client_builder();
         let client = client_builder
             .build()
@@ -192,17 +215,17 @@ impl RyHttpClient {
         )
     )]
     #[expect(clippy::too_many_arguments)]
-    pub fn get<'py>(
+    fn get<'py>(
         &'py self,
         py: Python<'py>,
         url: &Bound<'py, PyAny>,
-        body: Option<ryo3_bytes::PyBytes>,
+        body: Option<&Bound<'py, PyAny>>,
         headers: Option<PyHeadersLike>,
         query: Option<&Bound<'py, PyAny>>,
         json: Option<&Bound<'py, PyAny>>,
         form: Option<&Bound<'py, PyAny>>,
         multipart: Option<&Bound<'py, PyAny>>,
-        timeout: Option<&ryo3_std::PyDuration>,
+        timeout: Option<&PyDuration>,
         version: Option<HttpVersion>,
     ) -> PyResult<Bound<'py, PyAny>> {
         let opts = RequestKwargs {
@@ -241,17 +264,17 @@ impl RyHttpClient {
         )
     )]
     #[expect(clippy::too_many_arguments)]
-    pub fn post<'py>(
+    fn post<'py>(
         &'py self,
         py: Python<'py>,
         url: &Bound<'py, PyAny>,
-        body: Option<ryo3_bytes::PyBytes>,
+        body: Option<&Bound<'py, PyAny>>,
         headers: Option<PyHeadersLike>,
         query: Option<&Bound<'py, PyAny>>,
         json: Option<&Bound<'py, PyAny>>,
         form: Option<&Bound<'py, PyAny>>,
         multipart: Option<&Bound<'py, PyAny>>,
-        timeout: Option<&ryo3_std::PyDuration>,
+        timeout: Option<&PyDuration>,
         version: Option<HttpVersion>,
     ) -> PyResult<Bound<'py, PyAny>> {
         let opts = RequestKwargs {
@@ -290,17 +313,17 @@ impl RyHttpClient {
         )
     )]
     #[expect(clippy::too_many_arguments)]
-    pub fn put<'py>(
+    fn put<'py>(
         &'py self,
         py: Python<'py>,
         url: &Bound<'py, PyAny>,
-        body: Option<ryo3_bytes::PyBytes>,
+        body: Option<&Bound<'py, PyAny>>,
         headers: Option<PyHeadersLike>,
         query: Option<&Bound<'py, PyAny>>,
         json: Option<&Bound<'py, PyAny>>,
         form: Option<&Bound<'py, PyAny>>,
         multipart: Option<&Bound<'py, PyAny>>,
-        timeout: Option<&ryo3_std::PyDuration>,
+        timeout: Option<&PyDuration>,
         version: Option<HttpVersion>,
     ) -> PyResult<Bound<'py, PyAny>> {
         let opts = RequestKwargs {
@@ -339,17 +362,17 @@ impl RyHttpClient {
         )
     )]
     #[expect(clippy::too_many_arguments)]
-    pub fn patch<'py>(
+    fn patch<'py>(
         &'py self,
         py: Python<'py>,
         url: &Bound<'py, PyAny>,
-        body: Option<ryo3_bytes::PyBytes>,
+        body: Option<&Bound<'py, PyAny>>,
         headers: Option<PyHeadersLike>,
         query: Option<&Bound<'py, PyAny>>,
         json: Option<&Bound<'py, PyAny>>,
         form: Option<&Bound<'py, PyAny>>,
         multipart: Option<&Bound<'py, PyAny>>,
-        timeout: Option<&ryo3_std::PyDuration>,
+        timeout: Option<&PyDuration>,
         version: Option<HttpVersion>,
     ) -> PyResult<Bound<'py, PyAny>> {
         let opts = RequestKwargs {
@@ -388,17 +411,17 @@ impl RyHttpClient {
         )
     )]
     #[expect(clippy::too_many_arguments)]
-    pub fn delete<'py>(
+    fn delete<'py>(
         &'py self,
         py: Python<'py>,
         url: &Bound<'py, PyAny>,
-        body: Option<ryo3_bytes::PyBytes>,
+        body: Option<&Bound<'py, PyAny>>,
         headers: Option<PyHeadersLike>,
         query: Option<&Bound<'py, PyAny>>,
         json: Option<&Bound<'py, PyAny>>,
         form: Option<&Bound<'py, PyAny>>,
         multipart: Option<&Bound<'py, PyAny>>,
-        timeout: Option<&ryo3_std::PyDuration>,
+        timeout: Option<&PyDuration>,
         version: Option<HttpVersion>,
     ) -> PyResult<Bound<'py, PyAny>> {
         let opts = RequestKwargs {
@@ -437,17 +460,17 @@ impl RyHttpClient {
         )
     )]
     #[expect(clippy::too_many_arguments)]
-    pub fn head<'py>(
+    fn head<'py>(
         &'py self,
         py: Python<'py>,
         url: &Bound<'py, PyAny>,
-        body: Option<ryo3_bytes::PyBytes>,
+        body: Option<&Bound<'py, PyAny>>,
         headers: Option<PyHeadersLike>,
         query: Option<&Bound<'py, PyAny>>,
         json: Option<&Bound<'py, PyAny>>,
         form: Option<&Bound<'py, PyAny>>,
         multipart: Option<&Bound<'py, PyAny>>,
-        timeout: Option<&ryo3_std::PyDuration>,
+        timeout: Option<&PyDuration>,
         version: Option<HttpVersion>,
     ) -> PyResult<Bound<'py, PyAny>> {
         let opts = RequestKwargs {
@@ -486,17 +509,17 @@ impl RyHttpClient {
         )
     )]
     #[expect(clippy::too_many_arguments)]
-    pub fn options<'py>(
+    fn options<'py>(
         &'py self,
         py: Python<'py>,
         url: &Bound<'py, PyAny>,
-        body: Option<ryo3_bytes::PyBytes>,
+        body: Option<&Bound<'py, PyAny>>,
         headers: Option<PyHeadersLike>,
         query: Option<&Bound<'py, PyAny>>,
         json: Option<&Bound<'py, PyAny>>,
         form: Option<&Bound<'py, PyAny>>,
         multipart: Option<&Bound<'py, PyAny>>,
-        timeout: Option<&ryo3_std::PyDuration>,
+        timeout: Option<&PyDuration>,
         version: Option<HttpVersion>,
     ) -> PyResult<Bound<'py, PyAny>> {
         let opts = RequestKwargs {
@@ -536,18 +559,18 @@ impl RyHttpClient {
         )
     )]
     #[expect(clippy::too_many_arguments)]
-    pub fn fetch<'py>(
+    pub(crate) fn fetch<'py>(
         &'py self,
         py: Python<'py>,
         url: &Bound<'py, PyAny>,
         method: Option<ryo3_http::HttpMethod>,
-        body: Option<ryo3_bytes::PyBytes>,
+        body: Option<&Bound<'py, PyAny>>,
         headers: Option<PyHeadersLike>,
         query: Option<&Bound<'py, PyAny>>,
         json: Option<&Bound<'py, PyAny>>,
         form: Option<&Bound<'py, PyAny>>,
         multipart: Option<&Bound<'py, PyAny>>,
-        timeout: Option<&ryo3_std::PyDuration>,
+        timeout: Option<&PyDuration>,
         version: Option<HttpVersion>,
     ) -> PyResult<Bound<'py, PyAny>> {
         let method = method.map_or_else(|| Method::GET, |m| m.0);
@@ -565,7 +588,6 @@ impl RyHttpClient {
         };
 
         let req = self.build_request(opts)?;
-        debug!("reqwest-client-fetch: {:#?}", req);
         pyo3_async_runtimes::tokio::future_into_py(py, async move {
             req.send()
                 .await
@@ -590,18 +612,18 @@ impl RyHttpClient {
         )
     )]
     #[expect(clippy::too_many_arguments)]
-    pub fn __call__<'py>(
+    fn __call__<'py>(
         &'py self,
         py: Python<'py>,
         url: &Bound<'py, PyAny>,
         method: Option<ryo3_http::HttpMethod>,
-        body: Option<ryo3_bytes::PyBytes>,
+        body: Option<&Bound<'py, PyAny>>,
         headers: Option<PyHeadersLike>,
         query: Option<&Bound<'py, PyAny>>,
         json: Option<&Bound<'py, PyAny>>,
         form: Option<&Bound<'py, PyAny>>,
         multipart: Option<&Bound<'py, PyAny>>,
-        timeout: Option<&ryo3_std::PyDuration>,
+        timeout: Option<&PyDuration>,
         version: Option<HttpVersion>,
     ) -> PyResult<Bound<'py, PyAny>> {
         self.fetch(
@@ -619,14 +641,15 @@ impl<'py> IntoPyObject<'py> for &ClientConfig {
         let dict = PyDict::new(py);
         dict.set_item(intern!(py, "headers"), self.headers.clone())?;
         dict.set_item(intern!(py, "user_agent"), self.user_agent.clone())?;
-        dict.set_item(intern!(py, "timeout"), self.timeout.clone())?;
-        dict.set_item(intern!(py, "read_timeout"), self.read_timeout.clone())?;
-        dict.set_item(intern!(py, "connect_timeout"), self.connect_timeout.clone())?;
+        dict.set_item(intern!(py, "timeout"), self.timeout)?;
+        dict.set_item(intern!(py, "read_timeout"), self.read_timeout)?;
+        dict.set_item(intern!(py, "connect_timeout"), self.connect_timeout)?;
         dict.set_item(intern!(py, "gzip"), self.gzip)?;
         dict.set_item(intern!(py, "brotli"), self.brotli)?;
         dict.set_item(intern!(py, "deflate"), self.deflate)?;
         dict.set_item(intern!(py, "zstd"), self.zstd)?;
         dict.set_item(intern!(py, "http1_only"), self.http1_only)?;
+        dict.set_item(intern!(py, "hickory_dns"), self.hickory_dns)?;
         Ok(dict)
     }
 }
@@ -638,7 +661,8 @@ impl ClientConfig {
             .brotli(self.brotli)
             .deflate(self.deflate)
             .zstd(self.zstd)
-            .cookie_store(self.cookies);
+            .cookie_store(self.cookies)
+            .hickory_dns(self.hickory_dns);
         if let Some(user_agent) = &self.user_agent {
             client_builder = client_builder.user_agent(user_agent.clone());
         }
