@@ -1,24 +1,25 @@
 use pyo3::basic::CompareOp;
-use pyo3::exceptions::{PyOverflowError, PyTypeError, PyValueError, PyZeroDivisionError};
 use pyo3::prelude::PyAnyMethods;
-use pyo3::types::{PyDelta, PyTuple, PyType};
-use pyo3::{
-    Bound, FromPyObject, IntoPyObject, IntoPyObjectExt, PyAny, PyResult, Python, pyclass, pymethods,
+use pyo3::types::{PyDict, PyInt, PyTuple};
+use pyo3::{Bound, FromPyObject, IntoPyObjectExt, PyAny, PyResult, Python, pyclass, pymethods};
+use ryo3_macro_rules::{
+    py_overflow_err, py_overflow_error, py_type_err, py_value_err, py_zero_division_err,
 };
 use std::hash::{DefaultHasher, Hash, Hasher};
 use std::ops::{Div, Mul};
 use std::time::Duration;
 
+const NANOS_PER_SEC: u32 = 1_000_000_000;
 const SECS_PER_MINUTE: u64 = 60;
 const MINS_PER_HOUR: u64 = 60;
 const HOURS_PER_DAY: u64 = 24;
 const DAYS_PER_WEEK: u64 = 7;
-
 const MAX_DAYS: u64 = u64::MAX / (SECS_PER_MINUTE * MINS_PER_HOUR * HOURS_PER_DAY);
 const MAX_WEEKS: u64 = u64::MAX / (SECS_PER_MINUTE * MINS_PER_HOUR * HOURS_PER_DAY * DAYS_PER_WEEK);
 
-#[derive(Debug, Clone, PartialEq)]
-#[pyclass(name = "Duration", module = "ry.ryo3", frozen)]
+#[derive(Copy, Clone, PartialEq)]
+#[pyclass(name = "Duration", frozen)]
+#[cfg_attr(feature = "ry", pyo3(module = "ry.ryo3"))]
 pub struct PyDuration(pub Duration);
 
 impl From<Duration> for PyDuration {
@@ -28,31 +29,69 @@ impl From<Duration> for PyDuration {
 }
 
 impl PyDuration {
+    fn new(secs: u64, nanos: u32) -> PyResult<Self> {
+        if nanos < NANOS_PER_SEC {
+            Ok(Self(Duration::new(secs, nanos)))
+        } else {
+            let secs = secs
+                .checked_add(u64::from(nanos / NANOS_PER_SEC))
+                .ok_or_else(|| {
+                    py_overflow_error!("overflow; seconds part of Duration::new too large")
+                })?;
+            let nanos = nanos % NANOS_PER_SEC;
+            Ok(Self(Duration::new(secs, nanos)))
+        }
+    }
+
+    #[must_use]
+    pub fn inner(&self) -> &Duration {
+        &self.0
+    }
+
     fn try_from_secs_f32(secs: f32) -> PyResult<Self> {
-        Duration::try_from_secs_f32(secs)
-            .map(Self::from)
-            .map_err(|e| PyValueError::new_err(format!("{e}")))
+        if secs.is_nan() {
+            py_value_err!("invalid value: nan")
+        } else if secs.is_infinite() {
+            if secs.is_sign_negative() {
+                py_type_err!("negative duration")
+            } else {
+                py_overflow_err!("invalid value: inf")
+            }
+        } else if secs < 0.0 {
+            py_type_err!("negative duration")
+        } else {
+            Duration::try_from_secs_f32(secs)
+                .map(Self::from)
+                .map_err(|e| py_overflow_error!("{e}"))
+        }
     }
 
     fn try_from_secs_f64(secs: f64) -> PyResult<Self> {
-        Duration::try_from_secs_f64(secs)
-            .map(Self::from)
-            .map_err(|e| PyValueError::new_err(format!("{e}")))
+        if secs.is_nan() {
+            py_value_err!("invalid value: nan")
+        } else if secs.is_infinite() {
+            if secs.is_sign_negative() {
+                py_type_err!("negative duration")
+            } else {
+                py_overflow_err!("invalid value: inf")
+            }
+        } else if secs < 0.0 {
+            py_type_err!("negative duration")
+        } else {
+            Duration::try_from_secs_f64(secs)
+                .map(Self::from)
+                .map_err(|e| py_overflow_error!("{e}"))
+        }
     }
 }
 
 #[pymethods]
+#[expect(clippy::needless_pass_by_value)]
 impl PyDuration {
     #[new]
     #[pyo3(signature = (secs = 0, nanos = 0))]
     fn py_new(secs: u64, nanos: u32) -> PyResult<Self> {
-        let carry = nanos / 1_000_000_000;
-        let nanos = nanos % 1_000_000_000;
-
-        let secs = secs
-            .checked_add(u64::from(carry))
-            .ok_or_else(|| PyOverflowError::new_err("overflow in Duration::new"))?;
-        Ok(Self(Duration::new(secs, nanos)))
+        Self::new(secs, nanos)
     }
 
     fn __getnewargs__<'py>(&self, py: Python<'py>) -> PyResult<Bound<'py, PyTuple>> {
@@ -108,11 +147,7 @@ impl PyDuration {
     }
 
     fn __repr__(&self) -> String {
-        format!(
-            "Duration(secs={}, nanos={})",
-            self.0.as_secs(),
-            self.0.subsec_nanos()
-        )
+        format!("{self:?}")
     }
 
     fn __hash__(&self) -> u64 {
@@ -133,10 +168,6 @@ impl PyDuration {
         self.0.as_nanos()
     }
 
-    fn dbg(&self) -> String {
-        format!("Duration<{:?}>", self.0)
-    }
-
     // ========================================================================
     // MATHS/OPERATORS
     // ========================================================================
@@ -161,55 +192,80 @@ impl PyDuration {
         }
     }
 
-    fn __add__(&self, other: PyDurationComparable) -> PyResult<Self> {
-        let maybe_duration = match other {
-            PyDurationComparable::PyDuration(other) => self.0.checked_add(other.0),
-            PyDurationComparable::Duration(other) => self.0.checked_add(other),
-        };
-        if let Some(duration) = maybe_duration {
-            Ok(Self(duration))
+    fn __add__(&self, other: &Bound<'_, PyAny>) -> PyResult<Self> {
+        if let Ok(d) = other.cast_exact::<Self>() {
+            let rs_dur = d.get();
+            self.0
+                .checked_add(rs_dur.0)
+                .map(Self::from)
+                .ok_or_else(|| py_overflow_error!("overflow in Duration addition"))
+        } else if let Ok(d) = other.cast::<pyo3::types::PyDelta>() {
+            let rs_dur: Duration = d.extract()?;
+            self.0
+                .checked_add(rs_dur)
+                .map(Self::from)
+                .ok_or_else(|| py_overflow_error!("overflow in Duration addition"))
         } else {
-            Err(PyOverflowError::new_err("overflow in Duration addition"))
+            py_type_err!("unsupported operand type(s); must be Duration | datetime.timedelta")
         }
     }
 
-    fn __sub__(&self, other: PyDurationComparable) -> PyResult<Self> {
-        let maybe_duration = match other {
-            PyDurationComparable::PyDuration(other) => self.0.checked_sub(other.0),
-            PyDurationComparable::Duration(other) => self.0.checked_sub(other),
-        };
-        if let Some(duration) = maybe_duration {
-            Ok(Self(duration))
+    fn __radd__(&self, other: &Bound<'_, PyAny>) -> PyResult<Self> {
+        self.__add__(other)
+    }
+
+    fn __sub__(&self, other: &Bound<'_, PyAny>) -> PyResult<Self> {
+        if let Ok(d) = other.cast_exact::<Self>() {
+            let rs_dur = d.get();
+            self.0
+                .checked_sub(rs_dur.0)
+                .map(Self::from)
+                .ok_or_else(|| py_overflow_error!("overflow in Duration subtraction"))
+        } else if let Ok(d) = other.cast::<pyo3::types::PyDelta>() {
+            let rs_dur: Duration = d.extract()?;
+            self.0
+                .checked_sub(rs_dur)
+                .map(Self::from)
+                .ok_or_else(|| py_overflow_error!("overflow in Duration subtraction"))
         } else {
-            Err(PyOverflowError::new_err("overflow in Duration subtraction"))
+            py_type_err!("unsupported operand type(s); must be Duration | datetime.timedelta")
         }
     }
 
+    fn __rsub__(&self, other: &Bound<'_, PyAny>) -> PyResult<Self> {
+        self.__sub__(other)
+    }
+
     fn __truediv__<'py>(
         &self,
         py: Python<'py>,
-        other: PyDurationArithmeticDiv,
+        other: &Bound<'py, PyAny>,
     ) -> PyResult<Bound<'py, PyAny>> {
-        match other {
-            PyDurationArithmeticDiv::Int(other) => {
-                if other == 0 {
-                    return Err(PyZeroDivisionError::new_err("division by zero"));
-                }
-                self.0
-                    .checked_div(other)
-                    .map(Self::from)
-                    .ok_or_else(|| PyOverflowError::new_err("overflow in Duration division"))?
-                    .into_bound_py_any(py)
+        if let Ok(f) = other.cast::<PyInt>() {
+            let i = f.extract::<u32>()?;
+            if i == 0 {
+                return py_zero_division_err!("division by zero");
             }
-            PyDurationArithmeticDiv::Float(other) => self.div_f64(other)?.into_bound_py_any(py),
-            PyDurationArithmeticDiv::PyDuration(other) => {
-                let result = self.0.div_duration_f64(other.0);
-                Self::try_from_secs_f64(result)?.into_bound_py_any(py)
-            }
-            PyDurationArithmeticDiv::Duration(other) => {
-                let result = self.0.div_duration_f64(other);
-                Self::try_from_secs_f64(result)?.into_bound_py_any(py)
+            self.checked_div(i)
+                .ok_or_else(|| py_overflow_error!("overflow in Duration division"))?
+                .into_bound_py_any(py)
+        } else if let Ok(f) = other.cast::<pyo3::types::PyFloat>() {
+            let f = f.extract::<f64>()?;
+            self.div_f64(f).and_then(|d| d.into_bound_py_any(py))
+        } else if let Ok(d) = other.cast_exact::<Self>() {
+            let rs_dur = d.get();
+            self.div_duration_f64(rs_dur)?.into_bound_py_any(py)
+        } else if let Ok(d) = other.cast::<pyo3::types::PyDelta>() {
+            let rs_dur: Duration = d.extract()?;
+            if rs_dur.is_zero() {
+                py_zero_division_err!()
+            } else {
+                self.0.div_duration_f64(rs_dur).into_bound_py_any(py)
             }
+        } else {
+            py_type_err!(
+                "unsupported operand type(s); must be int | float | Duration | datetime.timedelta"
+            )
         }
     }
 
@@ -218,16 +274,18 @@ impl PyDuration {
             self.0
                 .checked_mul(i)
                 .map(Self::from)
-                .ok_or_else(|| PyOverflowError::new_err("overflow in Duration multiplication"))
+                .ok_or_else(|| py_overflow_error!("overflow in Duration multiplication"))
         } else if let Ok(f) = other.extract::<f64>() {
             self.mul_f64(f)
         } else {
-            return Err(PyTypeError::new_err(
-                "unsupported operand type(s); must be int | float",
-            ));
+            py_overflow_err!("unsupported operand type(s); must be int | float")
         }
     }
 
+    fn __rmul__(&self, other: &Bound<'_, PyAny>) -> PyResult<Self> {
+        self.__mul__(other)
+    }
+
     // ========================================================================
     // PROPERTIES/GETTERS
     // ========================================================================
@@ -251,6 +309,7 @@ impl PyDuration {
         self.0.as_secs() / 86400
     }
 
+    /// Return the number of seconds in the duration not counting days
     #[getter]
     fn seconds(&self) -> u64 {
         self.0.as_secs() % 86400
@@ -279,8 +338,8 @@ impl PyDuration {
     // ========================================================================
     // TO-CONVERSIONS
     // ========================================================================
-    fn as_secs(&self) -> f64 {
-        self.0.as_secs_f64()
+    fn as_secs(&self) -> u64 {
+        self.0.as_secs()
     }
 
     fn as_secs_f32(&self) -> f32 {
@@ -308,93 +367,109 @@ impl PyDuration {
     // ========================================================================
 
     /// Convert to python `datetime.timedelta`
+    #[expect(clippy::wrong_self_convention)]
     fn to_py(&self) -> Duration {
         self.0
     }
 
     /// Convert to python `datetime.timedelta`
-    fn to_pytimedelta<'py>(&self, py: Python<'py>) -> PyResult<Bound<'py, PyDelta>> {
-        self.0.into_pyobject(py)
+    #[expect(clippy::wrong_self_convention)]
+    fn to_pytimedelta(&self) -> Duration {
+        self.0
     }
 
     /// Convert from python `datetime.timedelta`
-    #[classmethod]
-    fn from_pytimedelta(_cls: &Bound<'_, PyType>, delta: Duration) -> Self {
+    #[staticmethod]
+    fn from_pytimedelta(delta: Duration) -> Self {
         Self(delta)
     }
 
+    // ========================================================================
+    // TO/FROM NUMBERS
+    // ========================================================================
+    #[expect(clippy::wrong_self_convention)]
+    fn to_dict<'py>(&self, py: Python<'py>) -> PyResult<Bound<'py, PyDict>> {
+        let dict = PyDict::new(py);
+        dict.set_item(interns::secs(py), self.0.as_secs())?;
+        dict.set_item(interns::nanos(py), self.0.subsec_nanos())?;
+        Ok(dict)
+    }
+
+    #[staticmethod]
+    fn from_dict(dict: &Bound<'_, PyDict>) -> PyResult<Self> {
+        let secs: u64 = dict.get_item(interns::secs(dict.py()))?.extract()?;
+        let nanos: u32 = dict.get_item(interns::nanos(dict.py()))?.extract()?;
+        Self::new(secs, nanos)
+    }
+
     // ========================================================================
     // FROM NUMBERS
     // ========================================================================
 
     /// Create a new `Duration` from the specified number of seconds.
-    #[classmethod]
-    fn from_secs(_cls: &Bound<'_, PyType>, secs: u64) -> Self {
+    #[staticmethod]
+    fn from_secs(secs: u64) -> Self {
         Self(Duration::from_secs(secs))
     }
 
-    #[classmethod]
-    fn from_millis(_cls: &Bound<'_, PyType>, millis: u64) -> Self {
+    #[staticmethod]
+    fn from_millis(millis: u64) -> Self {
         Self(Duration::from_millis(millis))
     }
 
-    #[classmethod]
-    fn from_micros(_cls: &Bound<'_, PyType>, micros: u64) -> Self {
+    #[staticmethod]
+    fn from_micros(micros: u64) -> Self {
         Self(Duration::from_micros(micros))
     }
 
-    #[classmethod]
-    fn from_nanos(_cls: &Bound<'_, PyType>, nanos: u64) -> Self {
+    #[staticmethod]
+    fn from_nanos(nanos: u64) -> Self {
         Self(Duration::from_nanos(nanos))
     }
 
-    #[classmethod]
-    fn from_hours(_cls: &Bound<'_, PyType>, hours: u64) -> PyResult<Self> {
+    #[staticmethod]
+    fn from_hours(hours: u64) -> PyResult<Self> {
         if hours > u64::MAX / (60 * 60) {
-            Err(PyOverflowError::new_err("overflow in Duration::from_hours"))
+            py_overflow_err!("overflow in Duration::from_hours")
         } else {
             Ok(Self(Duration::from_secs(hours * 60 * 60)))
         }
     }
 
-    #[classmethod]
-    fn from_mins(_cls: &Bound<'_, PyType>, mins: u64) -> PyResult<Self> {
+    #[staticmethod]
+    fn from_mins(mins: u64) -> PyResult<Self> {
         if mins > u64::MAX / 60 {
-            Err(PyOverflowError::new_err("overflow in Duration::from_mins"))
+            py_overflow_err!("overflow in Duration::from_mins")
         } else {
             Ok(Self(Duration::from_secs(mins * 60)))
         }
     }
 
-    #[classmethod]
-    fn from_days(_cls: &Bound<'_, PyType>, days: u64) -> PyResult<Self> {
-        if days > u64::MAX / MAX_DAYS {
-            Err(PyOverflowError::new_err(format!(
-                "overflow in Duration::from_days: {days} > {MAX_DAYS}"
-            )))
+    #[staticmethod]
+    fn from_days(days: u64) -> PyResult<Self> {
+        if days > MAX_DAYS {
+            py_overflow_err!("overflow in Duration::from_days: {days} > {MAX_DAYS}")
         } else {
             Ok(Self(Duration::from_secs(days * 60 * 60 * 24)))
         }
     }
 
-    #[classmethod]
-    fn from_weeks(_cls: &Bound<'_, PyType>, weeks: u64) -> PyResult<Self> {
+    #[staticmethod]
+    fn from_weeks(weeks: u64) -> PyResult<Self> {
         if weeks > u64::MAX / (MAX_WEEKS) {
-            Err(PyOverflowError::new_err(format!(
-                "overflow in Duration::from_weeks: {weeks} > {MAX_WEEKS}"
-            )))
+            py_overflow_err!("overflow in Duration::from_weeks: {weeks} > {MAX_WEEKS}")
         } else {
             Ok(Self(Duration::from_secs(weeks * 60 * 60 * 24 * 7)))
         }
     }
 
-    #[classmethod]
-    fn from_secs_f32(_cls: &Bound<'_, PyType>, secs: f32) -> PyResult<Self> {
+    #[staticmethod]
+    fn from_secs_f32(secs: f32) -> PyResult<Self> {
         Self::try_from_secs_f32(secs)
     }
 
-    #[classmethod]
-    fn from_secs_f64(_cls: &Bound<'_, PyType>, secs: f64) -> PyResult<Self> {
+    #[staticmethod]
+    fn from_secs_f64(secs: f64) -> PyResult<Self> {
         Self::try_from_secs_f64(secs)
     }
 
@@ -403,13 +478,13 @@ impl PyDuration {
     // ========================================================================
     #[pyo3(signature = (interval = None))]
     /// Sleep for the duration
-    pub fn sleep(&self, py: Python<'_>, interval: Option<u64>) -> PyResult<()> {
+    pub(crate) fn sleep(&self, py: Python<'_>, interval: Option<u64>) -> PyResult<()> {
         let interval = match interval {
             Some(interval) => {
                 if interval > 1000 {
-                    return Err(PyValueError::new_err(
-                        "interval must be less than or equal to 1000",
-                    ));
+                    return py_value_err!("interval must be less than or equal to 1000");
+                } else if interval == 0 {
+                    return py_value_err!("interval must be greater than 0");
                 }
                 interval
             }
@@ -478,44 +553,80 @@ impl PyDuration {
         self.0.checked_sub(other.0).map(Self::from)
     }
 
-    fn div_duration_f32(&self, other: &Self) -> f32 {
-        self.0.div_duration_f32(other.0)
+    fn div_duration_f32(&self, other: &Self) -> PyResult<f32> {
+        if other.0.is_zero() {
+            py_zero_division_err!()
+        } else {
+            Ok(self.0.div_duration_f32(other.0))
+        }
     }
 
-    fn div_duration_f64(&self, other: &Self) -> f64 {
-        self.0.div_duration_f64(other.0)
+    fn div_duration_f64(&self, other: &Self) -> PyResult<f64> {
+        if other.0.is_zero() {
+            py_zero_division_err!()
+        } else {
+            Ok(self.0.div_duration_f64(other.0))
+        }
     }
 
     fn div_f32(&self, n: f32) -> PyResult<Self> {
-        if n == 0.0 {
-            return Err(PyZeroDivisionError::new_err("division by zero"));
+        if n.abs() == 0.0 {
+            py_zero_division_err!()
+        } else if n.is_nan() {
+            py_value_err!("invalid value: nan")
+        } else if n.is_infinite() {
+            py_overflow_err!("invalid value: inf")
+        } else if n.is_sign_negative() {
+            py_type_err!("negative divisor")
+        } else {
+            let result = self.0.as_secs_f32().div(n);
+            Self::try_from_secs_f32(result)
         }
-        if n.is_sign_negative() || n.is_infinite() || n.is_nan() {
-            return Err(PyValueError::new_err("invalid value"));
-        }
-        let result = self.0.as_secs_f32().div(n);
-        Self::try_from_secs_f32(result)
     }
 
     fn div_f64(&self, n: f64) -> PyResult<Self> {
-        if n == 0.0 {
-            return Err(PyZeroDivisionError::new_err("division by zero"));
+        if n.abs() == 0.0 {
+            py_zero_division_err!()
+        } else if n.is_nan() {
+            py_value_err!("invalid value: nan")
+        } else if n.is_infinite() {
+            py_overflow_err!("invalid value: inf")
+        } else if n.is_sign_negative() {
+            py_type_err!("negative divisor")
+        } else {
+            let result = self.0.as_secs_f64().div(n);
+            Self::try_from_secs_f64(result)
         }
-        if n.is_sign_negative() || n.is_infinite() || n.is_nan() {
-            return Err(PyValueError::new_err("invalid value"));
-        }
-        let result = self.0.as_secs_f64().div(n);
-        Self::try_from_secs_f64(result)
     }
 
     fn mul_f32(&self, n: f32) -> PyResult<Self> {
-        let result = self.0.as_secs_f32().mul(n);
-        Self::try_from_secs_f32(result)
+        if n.abs() == 0.0 {
+            Ok(Self::from(Duration::ZERO))
+        } else if n.is_infinite() {
+            py_overflow_err!()
+        } else if n.is_nan() {
+            py_value_err!("invalid value: nan")
+        } else if n.is_sign_negative() {
+            py_type_err!("negative factor")
+        } else {
+            let result = self.0.as_secs_f32().mul(n);
+            Self::try_from_secs_f32(result)
+        }
     }
 
     fn mul_f64(&self, n: f64) -> PyResult<Self> {
-        let result = self.0.as_secs_f64().mul(n);
-        Self::try_from_secs_f64(result)
+        if n.abs() == 0.0 {
+            Ok(Self::from(Duration::ZERO))
+        } else if n.is_infinite() {
+            py_overflow_err!()
+        } else if n.is_nan() {
+            py_value_err!("invalid value: nan")
+        } else if n.is_sign_negative() {
+            py_type_err!("negative factor")
+        } else {
+            let result = self.0.as_secs_f64().mul(n);
+            Self::try_from_secs_f64(result)
+        }
     }
 
     fn saturating_add(&self, other: &Self) -> Self {
@@ -531,16 +642,31 @@ impl PyDuration {
     }
 }
 
+impl std::fmt::Debug for PyDuration {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(
+            f,
+            "Duration(secs={}, nanos={})",
+            self.0.as_secs(),
+            self.0.subsec_nanos()
+        )
+    }
+}
+
 #[derive(Debug, Clone, FromPyObject)]
 enum PyDurationComparable {
     PyDuration(PyDuration),
     Duration(Duration),
 }
 
-#[derive(Debug, Clone, FromPyObject)]
-enum PyDurationArithmeticDiv {
-    Int(u32), // matches Duration::checked_mul
-    Float(f64),
-    PyDuration(PyDuration),
-    Duration(Duration),
+// ----------------------------------------------------------------------------
+// py-string interns
+// ----------------------------------------------------------------------------
+
+mod interns {
+    use pyo3::prelude::*;
+    use ryo3_macro_rules::py_intern_fn;
+
+    py_intern_fn!(secs);
+    py_intern_fn!(nanos);
 }
