@@ -2,6 +2,7 @@
 use pyo3::exceptions::{PyOverflowError, PyRuntimeError, PyTypeError, PyValueError};
 use pyo3::types::{PyBytes, PyDict, PyModule, PyType};
 use pyo3::{IntoPyObjectExt, intern, prelude::*};
+use ryo3_pydantic::GetPydanticCoreSchemaCls;
 use ryo3_uuid::{CPythonUuid, PyUuid};
 use std::fmt::Write;
 use std::hash::{DefaultHasher, Hash, Hasher};
@@ -24,9 +25,11 @@ fn gen_new() -> PyResult<Ulid> {
         .map_err(|_| PyOverflowError::new_err("ULID-generator overflow"))
 }
 
-#[pyclass(name = "ULID", module = "ry.ulid", frozen, weakref)]
+#[pyclass(name = "ULID", frozen, weakref)]
+#[cfg_attr(feature = "ry", pyo3(module = "ry.ulid"))]
 #[cfg_attr(feature = "serde", derive(serde::Serialize), serde(transparent))]
-pub struct PyUlid(pub Ulid);
+#[derive(Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
+pub struct PyUlid(Ulid);
 
 #[cfg(feature = "serde")]
 impl<'de> serde::Deserialize<'de> for PyUlid {
@@ -39,7 +42,7 @@ impl<'de> serde::Deserialize<'de> for PyUlid {
 }
 
 impl PyUlid {
-    fn to_u128(&self) -> u128 {
+    fn to_u128(self) -> u128 {
         let b = self.0.to_bytes();
         u128::from_be_bytes(b)
     }
@@ -74,13 +77,13 @@ impl PyUlid {
     pub fn py_new(value: Option<Bound<'_, PyAny>>) -> PyResult<Self> {
         if let Some(value) = value {
             // IF IS BYTES
-            if let Ok(b) = value.downcast::<PyBytes>() {
+            if let Ok(b) = value.cast::<PyBytes>() {
                 let slice = b.as_bytes();
                 let b: [u8; 16] = slice
                     .try_into()
                     .map_err(|_| PyValueError::new_err("ULID must be exactly 16 bytes long"))?;
                 Ok(Self::from_bytes(b))
-            } else if let Ok(str) = value.downcast::<pyo3::types::PyString>() {
+            } else if let Ok(str) = value.cast::<pyo3::types::PyString>() {
                 let cs = str.to_str()?;
                 Self::from_string(cs)
             } else {
@@ -114,7 +117,7 @@ impl PyUlid {
     }
 
     fn __richcmp__(&self, other: &Bound<'_, PyAny>, op: pyo3::basic::CompareOp) -> PyResult<bool> {
-        if let Ok(pyint) = other.downcast::<pyo3::types::PyInt>() {
+        if let Ok(pyint) = other.cast::<pyo3::types::PyInt>() {
             let other: u128 = pyint.extract()?;
             match op {
                 pyo3::basic::CompareOp::Eq => Ok(self.to_u128() == other),
@@ -125,19 +128,19 @@ impl PyUlid {
                 pyo3::basic::CompareOp::Ge => Ok(self.to_u128() >= other),
             }
         } else if other.is_instance_of::<pyo3::types::PyString>() {
-            let s = other.downcast::<pyo3::types::PyString>()?;
+            let s = other.cast::<pyo3::types::PyString>()?;
             // visitor.visit_str(&s.to_cow()?)
             let cs = s.to_str()?;
             let this_str = self.0.to_string();
-            return match op {
+            match op {
                 pyo3::basic::CompareOp::Eq => Ok(this_str.as_str() == cs),
                 pyo3::basic::CompareOp::Ne => Ok(this_str.as_str() != cs),
                 pyo3::basic::CompareOp::Lt => Ok(this_str.as_str() < cs),
                 pyo3::basic::CompareOp::Le => Ok(this_str.as_str() <= cs),
                 pyo3::basic::CompareOp::Gt => Ok(this_str.as_str() > cs),
                 pyo3::basic::CompareOp::Ge => Ok(this_str.as_str() >= cs),
-            };
-        } else if let Ok(rs_ulid) = other.downcast::<Self>() {
+            }
+        } else if let Ok(rs_ulid) = other.cast::<Self>() {
             let other = rs_ulid.borrow().0;
             match op {
                 pyo3::basic::CompareOp::Eq => Ok(self.0 == other),
@@ -147,14 +150,14 @@ impl PyUlid {
                 pyo3::basic::CompareOp::Gt => Ok(self.0 > other),
                 pyo3::basic::CompareOp::Ge => Ok(self.0 >= other),
             }
-        } else if let Ok(pybytes) = other.downcast::<PyBytes>() {
+        } else if let Ok(pybytes) = other.cast::<PyBytes>() {
             let slice = pybytes.as_bytes();
             match slice.len() {
                 16 => {
                     let ulid = Ulid::from_bytes(
                         slice
                             .try_into()
-                            .expect("never to happen; checked lenght above"),
+                            .expect("never to happen; checked length above"),
                     );
                     match op {
                         pyo3::basic::CompareOp::Eq => Ok(self.0 == ulid),
@@ -188,6 +191,7 @@ impl PyUlid {
         Self(ulid)
     }
 
+    #[expect(clippy::wrong_self_convention)]
     fn to_bytes<'py>(&self, py: Python<'py>) -> Bound<'py, PyBytes> {
         self.__bytes__(py)
     }
@@ -249,18 +253,18 @@ impl PyUlid {
 
     #[staticmethod]
     fn from_timestamp(value: &Bound<'_, PyAny>) -> PyResult<Self> {
-        if let Ok(pyint) = value.downcast::<pyo3::types::PyInt>() {
+        if let Ok(pyint) = value.cast::<pyo3::types::PyInt>() {
             let i = pyint
                 .extract::<u64>()
                 .map_err(|_| PyOverflowError::new_err("value gt u64::MAX"))?;
             Self::from_timestamp_milliseconds(i)
-        } else if let Ok(pyfloat) = value.downcast::<pyo3::types::PyFloat>() {
+        } else if let Ok(pyfloat) = value.cast::<pyo3::types::PyFloat>() {
             let f = pyfloat.extract::<f64>()?;
             Self::from_timestamp_seconds(f)
         } else {
-            return Err(PyTypeError::new_err(
+            Err(PyTypeError::new_err(
                 "Expected a float (seconds) or int (ms) for timestamp",
-            ));
+            ))
         }
     }
 
@@ -270,18 +274,19 @@ impl PyUlid {
     }
 
     #[staticmethod]
-    #[expect(clippy::needless_pass_by_value)] // must do bc of pyo3
     fn from_uuid(uu: UuidLike) -> Self {
         let uu = uu.0;
         let ul = Ulid::from_bytes(*uu.as_bytes());
         Self(ul)
     }
 
+    #[expect(clippy::wrong_self_convention)]
     fn to_uuid(&self) -> PyUuid {
         let b = self.0.to_bytes();
         ryo3_uuid::PyUuid::from(Uuid::from_bytes(b))
     }
 
+    #[expect(clippy::wrong_self_convention)]
     fn to_uuid4(&self) -> PyUuid {
         let mut b = uuid::Builder::from_u128(self.to_u128());
         b.set_version(uuid::Version::Random);
@@ -290,7 +295,7 @@ impl PyUlid {
 
     #[staticmethod]
     fn parse(other: &Bound<'_, PyAny>) -> PyResult<Self> {
-        if let Ok(pyint) = other.downcast::<pyo3::types::PyInt>() {
+        if let Ok(pyint) = other.cast::<pyo3::types::PyInt>() {
             let i = pyint.extract::<u128>()?;
             if let Ok(smaller_int) = u64::try_from(i) {
                 let ulid = Ulid::from_datetime(
@@ -301,7 +306,7 @@ impl PyUlid {
             // If the integer is too large, we treat it as a ULID.
             Ok(Self::from_int(i))
         } else if other.is_instance_of::<pyo3::types::PyString>() {
-            let s = other.downcast::<pyo3::types::PyString>()?;
+            let s = other.cast::<pyo3::types::PyString>()?;
             let cs = s.to_str()?;
             // uuid string length
             match cs.len() {
@@ -309,48 +314,42 @@ impl PyUlid {
                     let uu = Uuid::parse_str(cs)
                         .map_err(|e| PyValueError::new_err(format!("Invalid UUID string: {e}")))?;
                     let ul = Ulid::from_bytes(*uu.as_bytes());
-                    return Ok(Self(ul));
+                    Ok(Self(ul))
                 }
                 26 => {
                     let ulid = Ulid::from_string(cs)
                         .map_err(|e| PyValueError::new_err(format!("Invalid ULID string: {e}")))?;
-                    return Ok(Self(ulid));
-                }
-                32 => {
-                    return Self::from_hex(cs);
-                }
-                _ => {
-                    return Err(PyValueError::new_err(format!(
-                        "Cannot parse ULID from string of length {}",
-                        cs.len()
-                    )));
+                    Ok(Self(ulid))
                 }
+                32 => Self::from_hex(cs),
+                _ => Err(PyValueError::new_err(format!(
+                    "Cannot parse ULID from string of length {}",
+                    cs.len()
+                ))),
             }
         }
         // has to go through `isinstance` apparatus
         else if other.is_instance_of::<pyo3::types::PyFloat>() {
             let f = other.extract::<f64>()?;
-            return Self::from_timestamp_seconds(f);
-        } else if let Ok(rs_ulid) = other.downcast::<Self>() {
+            Self::from_timestamp_seconds(f)
+        } else if let Ok(rs_ulid) = other.cast::<Self>() {
             let inner = rs_ulid.borrow().0;
-            return Ok(Self(inner));
+            Ok(Self(inner))
         } else if other.is_instance_of::<PyBytes>() {
-            let pybytes = other.downcast::<PyBytes>()?;
+            let pybytes = other.cast::<PyBytes>()?;
             let b = pybytes.extract::<[u8; 16]>()?;
-            return Ok(Self::from_bytes(b));
-        } else if let Ok(py_uuid) = other.downcast::<PyUuid>() {
-            return Ok(Self::from_uuid(UuidLike(py_uuid.borrow().0)));
+            Ok(Self::from_bytes(b))
+        } else if let Ok(py_uuid) = other.cast::<PyUuid>() {
+            return Ok(Self::from_uuid(UuidLike(*py_uuid.borrow().get())));
         } else if let Ok(c_uuid) = other.extract::<CPythonUuid>() {
-            return Ok(Self::from_uuid(UuidLike(c_uuid.into())));
+            Ok(Self::from_uuid(UuidLike(c_uuid.into())))
         } else if let Ok(dt) = other.extract::<SystemTime>() {
-            return Ok(Self::from_datetime(dt));
+            Ok(Self::from_datetime(dt))
         } else {
             let other_type = other.get_type();
             let other_type_name = other_type
                 .name()
-                .map(|e| e.as_borrowed().to_string())
-                .unwrap_or(String::from("Unknown"));
-
+                .map_or_else(|_| String::from("Unknown"), |e| e.as_borrowed().to_string());
             Err(PyTypeError::new_err(format!(
                 "Cannot parse ULID from type {other_type_name}"
             )))
@@ -400,62 +399,9 @@ impl PyUlid {
     fn __get_pydantic_core_schema__<'py>(
         cls: &Bound<'py, PyType>,
         source: &Bound<'py, PyAny>,
-        _handler: &Bound<'py, PyAny>,
+        handler: &Bound<'py, PyAny>,
     ) -> PyResult<Bound<'py, PyAny>> {
-        let py = source.py();
-        let core_schema = py.import(intern!(py, "pydantic_core"))?;
-        let core_schema = core_schema.getattr(intern!(py, "core_schema"))?;
-
-        // oy vey this is hideous, but it works
-        let str_schema_kwargs = PyDict::new(py);
-        str_schema_kwargs.set_item(intern!(py, "pattern"), intern!(py, r"[A-Z0-9]{26}"))?;
-        str_schema_kwargs.set_item(intern!(py, "min_length"), 26)?;
-        str_schema_kwargs.set_item(intern!(py, "max_length"), 26)?;
-
-        // more hideousness
-        let bytes_schema_kwargs = PyDict::new(py);
-        bytes_schema_kwargs.set_item(intern!(py, "min_length"), 16)?;
-        bytes_schema_kwargs.set_item(intern!(py, "max_length"), 16)?;
-
-        let union_schema = core_schema.call_method1(
-            intern!(py, "union_schema"),
-            (vec![
-                core_schema
-                    .call_method1(intern!(py, "is_instance_schema"), (py.get_type::<Self>(),))?,
-                core_schema.call_method1(
-                    intern!(py, "no_info_plain_validator_function"),
-                    (py.get_type::<Self>(),),
-                )?,
-                core_schema.call_method(intern!(py, "str_schema"), (), Some(&str_schema_kwargs))?,
-                core_schema.call_method(
-                    intern!(py, "bytes_schema"),
-                    (),
-                    Some(&bytes_schema_kwargs),
-                )?,
-            ],),
-        )?;
-
-        let to_string_ser_schema_kwargs = PyDict::new(py);
-        to_string_ser_schema_kwargs
-            .set_item(intern!(py, "when_used"), intern!(py, "json-unless-none"))?;
-        let to_string_ser_schema = core_schema.call_method(
-            intern!(py, "to_string_ser_schema"),
-            (),
-            Some(&to_string_ser_schema_kwargs),
-        )?;
-
-        let pydantic_validate = cls.getattr(intern!(py, "_pydantic_validate"))?;
-
-        let no_info_wrap_validator_function_kwargs = PyDict::new(py);
-        no_info_wrap_validator_function_kwargs
-            .set_item(intern!(py, "serialization"), to_string_ser_schema)?;
-        let no_info_wrap_validator_function = core_schema.call_method(
-            intern!(py, "no_info_wrap_validator_function"),
-            (pydantic_validate, union_schema),
-            Some(&no_info_wrap_validator_function_kwargs),
-        )?;
-
-        Ok(no_info_wrap_validator_function)
+        Self::get_pydantic_core_schema(cls, source, handler)
     }
 
     #[classmethod]
@@ -465,19 +411,38 @@ impl PyUlid {
         handler: &Bound<'py, PyAny>,
     ) -> PyResult<Bound<'py, PyAny>> {
         let py = value.py();
-        let ulid = if let Ok(pyint) = value.downcast::<pyo3::types::PyInt>() {
+        let ulid = if let Ok(pyint) = value.cast::<pyo3::types::PyInt>() {
             cls.call_method1(intern!(py, "from_int"), (pyint,))
-        } else if let Ok(pystr) = value.downcast::<pyo3::types::PyString>() {
+        } else if let Ok(pystr) = value.cast::<pyo3::types::PyString>() {
             cls.call_method1(intern!(py, "from_str"), (pystr,))
-        } else if let Ok(pyulid) = value.downcast::<Self>() {
+        } else if let Ok(pyulid) = value.cast::<Self>() {
             pyulid.into_bound_py_any(py)
-        } else if let Ok(pybytes) = value.downcast::<PyBytes>() {
+        } else if let Ok(pybytes) = value.cast::<PyBytes>() {
             cls.call_method1(intern!(py, "from_bytes"), (pybytes,))
         } else {
             Err(PyTypeError::new_err("Unrecognized format for ULID"))
         }?;
         handler.call1((ulid,))
     }
+
+    #[classmethod]
+    fn _pydantic_validate_strict<'py>(
+        cls: &Bound<'py, PyType>,
+        value: &Bound<'py, PyAny>,
+        handler: &Bound<'py, PyAny>,
+    ) -> PyResult<Bound<'py, PyAny>> {
+        let py = value.py();
+        let ulid = if let Ok(pystr) = value.cast::<pyo3::types::PyString>() {
+            cls.call_method1(intern!(py, "from_str"), (pystr,))
+        } else if let Ok(pyulid) = value.cast::<Self>() {
+            pyulid.into_bound_py_any(py)
+        } else {
+            Err(PyValueError::new_err(
+                "Unrecognized format for ULID (strict)",
+            ))
+        }?;
+        handler.call1((ulid,))
+    }
 }
 
 impl From<Ulid> for PyUlid {
@@ -486,12 +451,13 @@ impl From<Ulid> for PyUlid {
     }
 }
 
-pub struct UuidLike(pub(crate) Uuid);
+#[derive(Clone, Copy)]
+struct UuidLike(pub(crate) Uuid);
 
 impl FromPyObject<'_> for UuidLike {
     fn extract_bound(obj: &Bound<'_, PyAny>) -> PyResult<Self> {
-        if let Ok(uuid_like) = obj.downcast::<PyUuid>() {
-            return Ok(Self(uuid_like.borrow().0));
+        if let Ok(uuid_like) = obj.cast::<PyUuid>() {
+            return Ok(Self(*uuid_like.borrow().get()));
         } else if let Ok(py_uuid) = obj.extract::<CPythonUuid>() {
             return Ok(Self(py_uuid.into()));
         }
@@ -499,6 +465,102 @@ impl FromPyObject<'_> for UuidLike {
     }
 }
 
+impl GetPydanticCoreSchemaCls for PyUlid {
+    fn get_pydantic_core_schema<'py>(
+        cls: &Bound<'py, PyType>,
+        source: &Bound<'py, PyAny>,
+        _handler: &Bound<'py, PyAny>,
+    ) -> PyResult<Bound<'py, PyAny>> {
+        use ryo3_pydantic::interns;
+        let py = source.py();
+        // let core_schema = py.import(intern!(py, "pydantic_core.core_schema"))?;
+        let core_schema = ryo3_pydantic::core_schema(py)?;
+
+        // let core_schema = core_schema.getattr(intern!(py, "core_schema"))?;
+
+        // oy vey this is hideous, but it works
+        let str_schema_kwargs = PyDict::new(py);
+        str_schema_kwargs.set_item(interns::pattern(py), intern!(py, r"[A-Z0-9]{26}"))?;
+        str_schema_kwargs.set_item(interns::min_length(py), 26)?;
+        str_schema_kwargs.set_item(interns::max_length(py), 26)?;
+
+        // more hideousness
+        let bytes_schema_kwargs = PyDict::new(py);
+        bytes_schema_kwargs.set_item(interns::min_length(py), 16)?;
+        bytes_schema_kwargs.set_item(interns::max_length(py), 16)?;
+
+        // actual validator functions
+        let pydantic_validate = cls.getattr(interns::_pydantic_validate(py))?;
+        let pydantic_validate_strict = cls.getattr(interns::_pydantic_validate_strict(py))?;
+
+        let to_string_ser_schema_kwargs = PyDict::new(py);
+        to_string_ser_schema_kwargs
+            .set_item(interns::when_used(py), interns::json_unless_none(py))?;
+        let to_string_ser_schema = core_schema.call_method(
+            interns::to_string_ser_schema(py),
+            (),
+            Some(&to_string_ser_schema_kwargs),
+        )?;
+
+        let no_info_wrap_validator_function_kwargs = PyDict::new(py);
+        no_info_wrap_validator_function_kwargs
+            .set_item(interns::serialization(py), &to_string_ser_schema)?;
+
+        // LAX union schema (allows ULID, string, bytes)
+        let lax_union_schema = core_schema.call_method1(
+            interns::union_schema(py),
+            (vec![
+                core_schema
+                    .call_method1(interns::is_instance_schema(py), (py.get_type::<Self>(),))?,
+                core_schema.call_method1(
+                    interns::no_info_plain_validator_function(py),
+                    (py.get_type::<Self>(),),
+                )?,
+                core_schema.call_method(interns::str_schema(py), (), Some(&str_schema_kwargs))?,
+                core_schema.call_method(
+                    interns::bytes_schema(py),
+                    (),
+                    Some(&bytes_schema_kwargs),
+                )?,
+            ],),
+        )?;
+
+        let strict_union = core_schema.call_method1(
+            interns::union_schema(py),
+            (vec![
+                core_schema
+                    .call_method1(interns::is_instance_schema(py), (py.get_type::<Self>(),))?,
+                core_schema.call_method(
+                    interns::str_schema(py),
+                    (),
+                    Some(&str_schema_kwargs), // still allow canonical string
+                )?,
+            ],),
+        )?;
+
+        let strict_schema = core_schema.call_method(
+            interns::no_info_wrap_validator_function(py),
+            (pydantic_validate_strict, strict_union),
+            Some(&no_info_wrap_validator_function_kwargs),
+        )?;
+
+        let ulid_schema_kwargs = PyDict::new(py);
+        ulid_schema_kwargs.set_item(interns::serialization(py), &to_string_ser_schema)?;
+
+        let lax_schema = core_schema.call_method(
+            interns::no_info_wrap_validator_function(py),
+            (pydantic_validate, lax_union_schema),
+            Some(&no_info_wrap_validator_function_kwargs),
+        )?;
+        let ulid_schema = core_schema.call_method(
+            interns::lax_or_strict_schema(py),
+            (lax_schema, strict_schema),
+            Some(&ulid_schema_kwargs),
+        )?;
+        Ok(ulid_schema)
+    }
+}
+
 pub fn pymod_add(m: &Bound<'_, PyModule>) -> PyResult<()> {
     m.add_class::<PyUlid>()?;
     Ok(())
