@@ -6,6 +6,7 @@ from ry.ryo3 import (
     __authors__,
     __build_profile__,
     __build_timestamp__,
+    __opt_level__,
     __target__,
     __version__,
 )
@@ -15,11 +16,12 @@ __all__ = (
     "__build_profile__",
     "__build_timestamp__",
     "__description__",
+    "__opt_level__",
     "__pkgroot__",
     "__target__",
     "__title__",
     "__version__",
 )
 __title__ = "ry"
-__description__ = "ry = rust + python - most of the letters"
+__description__ = "ry = rust + python"
 __pkgroot__ = __file__.replace("__about__.py", "").rstrip("/\\")
