@@ -6,6 +6,29 @@ import pytest
 
 import ry
 
+JIFF_UNITS = [
+    "nanosecond",
+    "microsecond",
+    "millisecond",
+    "second",
+    "minute",
+    "hour",
+    "day",
+    "month",
+    "year",
+]
+
+JIFF_ROUND_MODES = [
+    "ceil",
+    "floor",
+    "expand",
+    "trunc",
+    "half-ceil",
+    "half-floor",
+    "half-expand",
+    "half-trunc",
+    "half-even",
+]
 # ====================
 # Zoned
 # ====================
@@ -120,7 +143,7 @@ class TestOffset:
 
 def test_zoned() -> None:
     zdt = ry.date(2020, 8, 26).at(6, 27, 0, 0).in_tz("America/New_York")
-    assert zdt.string() == "2020-08-26T06:27:00-04:00[America/New_York]"
+    assert str(zdt) == "2020-08-26T06:27:00-04:00[America/New_York]"
 
     zdt_fields = {
         "tz": str(zdt.timezone),
@@ -168,7 +191,7 @@ def test_zoned() -> None:
         "nanosecond": 0,
     }
     assert dt_fields == dt_dictionary
-    assert ry_datetime.asdict() == {
+    assert ry_datetime.to_dict() == {
         "year": 2020,
         "month": 8,
         "day": 26,
@@ -194,7 +217,7 @@ def test_zoned() -> None:
     }
 
     expected_time_dict = {"hour": 6, "minute": 27, "second": 0, "nanosecond": 0}
-    assert ry_time.asdict() == expected_time_dict
+    assert ry_time.to_dict() == expected_time_dict
 
 
 # ====================
@@ -207,12 +230,12 @@ class TestTimeSpan:
         zdt1 = ry.date(2020, 8, 26).at(6, 27, 0, 0).in_tz("America/New_York")
         zdt2 = ry.date(2023, 12, 31).at(18, 30, 0, 0).in_tz("America/New_York")
         span = zdt2 - zdt1
-        assert span.string() == "PT29341H3M"
+        assert str(span) == "PT29341H3M"
         span_negated = -span
-        assert span_negated.string() == "-PT29341H3M"
+        assert str(span_negated) == "-PT29341H3M"
 
         span_inverted = ~span
-        assert span_inverted.string() == "-PT29341H3M"
+        assert str(span_inverted) == "-PT29341H3M"
 
     def test_span_2_duration(self) -> None:
         zdt1 = ry.date(2020, 8, 26).at(6, 27, 0, 0).in_tz("America/New_York")
@@ -291,49 +314,22 @@ class TestTimeSpanProperties:
         assert self.ts.nanoseconds == 5_000_000
 
 
-# ====================
-# round mode
-# ====================
-
-JIFF_UNITS = [
-    "nanosecond",
-    "microsecond",
-    "millisecond",
-    "second",
-    "minute",
-    "hour",
-    "day",
-    "month",
-    "year",
-]
-
-JIFF_ROUND_MODES = [
-    "ceil",
-    "floor",
-    "expand",
-    "trunc",
-    "half_ceil",
-    "half_floor",
-    "half_expand",
-    "half_trunc",
-    "half_even",
-]
-
-
 class TestDateTime:
     d = ry.date(2020, 8, 26).at(6, 27, 0, 0)
 
     def test_datetime_round_options(self) -> None:
         default = ry.DateTimeRound()
         expected_default_string = (
-            'DateTimeRound(smallest="nanosecond", mode="half_expand", increment=1)'
+            'DateTimeRound(smallest="nanosecond", mode="half-expand", increment=1)'
         )
         assert str(default) == expected_default_string
 
         for unit, mode in it.product(JIFF_UNITS, JIFF_ROUND_MODES):
             options = ry.DateTimeRound(smallest=unit, mode=mode, increment=1)  # type: ignore[arg-type]
 
-            options_chained = ry.DateTimeRound().smallest(unit).mode(mode).increment(1)  # type: ignore[arg-type]
+            options_chained = (
+                ry.DateTimeRound()._smallest(unit)._mode(mode)._increment(1)  # type: ignore[arg-type]
+            )
             expected_string = (
                 f'DateTimeRound(smallest="{unit}", mode="{mode}", increment=1)'
             )
@@ -348,7 +344,7 @@ class TestDateTime:
 class TestTimespanFunction:
     def test_timespan_fn(self) -> None:
         ts = ry.timespan(weeks=1)
-        assert ts.string() == "P1W"
+        assert str(ts) == "P1W"
 
     def test_timespan_overflow(self) -> None:
         max_i64 = 9_223_372_036_854_775_807
@@ -428,7 +424,7 @@ class TestTzOffset:
     def test_checked_add(self) -> None:
         offset = ry.Offset.from_hours(-8)
         span = ry.timespan(hours=1)
-        assert offset.checked_add(span) == ry.Offset.from_hours(-7)
+        assert offset.add(span) == ry.Offset.from_hours(-7)
         signed_duration = span.to_signed_duration(
             ry.Date(
                 year=2024,
@@ -436,20 +432,20 @@ class TestTzOffset:
                 day=13,  # OOOOH friday the 13th
             )
         )
-        assert offset.checked_add(signed_duration) == ry.Offset.from_hours(-7)
+        assert offset.add(signed_duration) == ry.Offset.from_hours(-7)
         duration = ry.Duration(secs=3600)
-        assert offset.checked_add(duration) == ry.Offset.from_hours(-7)
+        assert offset.add(duration) == ry.Offset.from_hours(-7)
 
     def test_checked_sub(self) -> None:
         offset = ry.Offset.from_hours(-8)
         span = ry.timespan(hours=1)
-        assert offset.checked_sub(span) == ry.Offset.from_hours(-9)
+        assert offset.sub(span) == ry.Offset.from_hours(-9)
         signed_duration = span.to_signed_duration(
             ry.Date(year=2024, month=12, day=13)  # OOOOH friday the 13th (again)
         )
-        assert offset.checked_sub(signed_duration) == ry.Offset.from_hours(-9)
+        assert offset.sub(signed_duration) == ry.Offset.from_hours(-9)
         duration = ry.Duration(secs=3600)
-        assert offset.checked_sub(duration) == ry.Offset.from_hours(-9)
+        assert offset.sub(duration) == ry.Offset.from_hours(-9)
 
     def test_saturating_add(self) -> None:
         offset = ry.Offset.from_hours(25)
@@ -545,19 +541,19 @@ class TestParse:
     zdt = ry.date(2020, 8, 26).at(6, 27, 0, 0).in_tz("America/New_York")
 
     def test_parse_date(self) -> None:
-        parsed_date = ry.Date.parse(self.d.string())
+        parsed_date = ry.Date.parse(str(self.d))
         assert parsed_date == self.d
 
     def test_parse_datetime(self) -> None:
-        parsed_datetime = ry.DateTime.parse(self.dt.string())
+        parsed_datetime = ry.DateTime.from_str(str(self.dt))
         assert parsed_datetime == self.dt
 
     def test_parse_time(self) -> None:
-        parsed_time = ry.Time.parse(self.t.string())
+        parsed_time = ry.Time.parse(str(self.t))
         assert parsed_time == self.t
 
     def test_parse_zoned_datetime(self) -> None:
-        parsed_zdt = ry.ZonedDateTime.parse(self.zdt.string())
+        parsed_zdt = ry.ZonedDateTime.parse(str(self.zdt))
         assert parsed_zdt == self.zdt
         assert parsed_zdt.timezone == self.zdt.timezone
         assert parsed_zdt.date() == self.zdt.date()
@@ -578,4 +574,4 @@ class TestJiffFunctions:
         assert isinstance(zdt, ry.ZonedDateTime)
         assert zdt.date() == ry.date(2020, 2, 29)
         assert zdt.time() == ry.time(12, 30, 45)
-        assert zdt.string() == "2020-02-29T12:30:45-08:00[America/Los_Angeles]"
+        assert str(zdt) == "2020-02-29T12:30:45-08:00[America/Los_Angeles]"
