@@ -1,9 +1,12 @@
 from __future__ import annotations
 
-from pathlib import Path
+from typing import TYPE_CHECKING
 
 import ry
 
+if TYPE_CHECKING:
+    from pathlib import Path
+
 
 def test_is_same_file(tmp_path: Path) -> None:
     a = tmp_path / "a"
